# Edit this configuration file to define what should be installed on
# your system.  Help is available in the configuration.nix(5) man page
# and in the NixOS manual (accessible by running ‘nixos-help’).

{ inputs, config, pkgs, ... }:

let python-packages = ps: with ps; [ ];
in rec {
  imports = [
    ./hardware-configuration.nix
    inputs.home-manager.nixosModules.home-manager
  ];

  home-manager = {
    extraSpecialArgs = { inherit inputs; };
    useGlobalPkgs = true;
    useUserPackages = true;
    users.julian = import ./home.nix;
  };

 nix.buildMachines = [ {
	 hostName = "192.168.3.118";
	 system = "x86_64-linux";
	 protocol = "ssh";
        # if the builder supports building for multiple architectures, 
        # replace the previous line by, e.g.
        # systems = ["x86_64-linux" "aarch64-linux"];
	 maxJobs = 4;
	 speedFactor = 3;
	 supportedFeatures = [ "nixos-test" "benchmark" "big-parallel" "kvm" ];
	 mandatoryFeatures = [ ];
}] ;
	nix.distributedBuilds = true;
       # optional, useful when the builder has a faster internet connection than yours
	nix.extraOptions = ''
	  builders-use-substitutes = true
	'';

  # Bootloader
  # Use this for simple nix boot menu, if no dual boot required
  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;
  boot.loader.efi.efiSysMountPoint = "/boot/efi";

  boot.supportedFilesystems = [ "btrfs" "ntfs" "nfs" "cifs" ];

  networking.hostName = "nixos"; # Define your hostname.
  # networking.wireless.enable = true;  # Enables wireless support via wpa_supplicant.

  # Configure network proxy if necessary
  # networking.proxy.default = "http://user:password@proxy:port/";
  # networking.proxy.noProxy = "127.0.0.1,localhost,internal.domain";

  # Enable networking
  networking.networkmanager.enable = true;

  # Set your time zone.
  time.timeZone = "Europe/Berlin";

  # Set location used by redshift
  location.provider = "manual";
  location.latitude = 47.92;
  location.longitude = 10.12;

  # Select internationalisation properties.
  i18n.defaultLocale = "en_US.UTF-8";

  i18n.extraLocaleSettings = {
    LC_ADDRESS = "de_DE.UTF-8";
    LC_IDENTIFICATION = "de_DE.UTF-8";
    LC_MEASUREMENT = "de_DE.UTF-8";
    LC_MONETARY = "de_DE.UTF-8";
    LC_NAME = "de_DE.UTF-8";
    LC_NUMERIC = "en_US.UTF-8";
    LC_PAPER = "de_DE.UTF-8";
    LC_TELEPHONE = "de_DE.UTF-8";
    LC_TIME = "de_DE.UTF-8";
  };

  nix.settings.auto-optimise-store = true;
  nix.settings.experimental-features = [ "nix-command" "flakes" ];

  # Setup binary caches
  nix.settings = {
  	substituters = [
	"https://nix-community.cachix.org"
	"https://cache.nixos.org/"
	];
	trusted-public-keys = [
	"nix-community.cachix.org-1:mB9FSh9qf2dCimDSUo8Zy7bkq5CX+/rkCWyvRCYg3Fs="
	];
  };

  # Enable the X11 windowing system.
  services.xserver.enable = true;
  services.xserver.videoDrivers = [ "nvidia" ];
  hardware.nvidia.package =
    config.boot.kernelPackages.nvidiaPackages.legacy_470;
  hardware.opengl.enable = true;
  # hardware.nvidia.modesetting.enable = true; # for wayland

  nix.gc = {
    automatic = true;
    dates = "weekly";
    options = "--delete-older-than 30d";
  };

  # Enable the XFCE Desktop Environment.
  services.xserver.displayManager.lightdm.enable = true;
  services.xserver.desktopManager = {
    xterm.enable = false;
    xfce = {
      enable = true;
      # noDesktop = true;
      # enableXfwm = false;
    };
  };
  services.xserver.displayManager.defaultSession = "none+i3";
  services.xserver.windowManager.i3.enable = true;
  services.xserver.windowManager.i3.package = pkgs.i3-gaps;
  services.xserver.windowManager.i3.extraPackages = with pkgs; [
    rofi
    i3lock
    i3status-rust
    nitrogen
  ];

  # Configure keymap in X11
  services.xserver = {
    layout = "de";
    xkbVariant = "";
  };

  # Configure console keymap
  console.keyMap = "de";

  # Enable CUPS to print documents.
  # hardware.sane.enable = true;
  services.printing.enable = true;
  services.printing.browsing = true;
  # services.avahi.enable = true;
  # services.avahi.nssmdns = true;
  services.printing.drivers = with pkgs; [ gutenprint ];

  # Enable sound with pipewire.
  sound.enable = true;
  hardware.pulseaudio.enable = false;
  security.rtkit.enable = true;
  services.pipewire = {
    enable = true;
    alsa.enable = true;
    alsa.support32Bit = true;
    pulse.enable = true;
    # If you want to use JACK applications, uncomment this
    #jack.enable = true;

    # use the example session manager (no others are packaged yet so this is enabled by default,
    # no need to redefine it in your config for now)
    #media-session.enable = true;
  };

  # Enable touchpad support (enabled default in most desktopManager).
  # services.xserver.libinput.enable = true;

  # Define a user account. Don't forget to set a password with ‘passwd’.
  users.users.julian = {
    isNormalUser = true;
    description = "Julian";
    uid = 1000;
    group = "julian";
    shell = pkgs.zsh;
    extraGroups = [ "networkmanager" "wheel" "docker" ];
    packages = with pkgs; [
      hunspell
      hunspellDicts.de_DE
      hunspellDicts.en_US
      makemkv
      audacity
      gnome.cheese
      zoom-us
      qutebrowser

      unetbootin

      # For SSS exercises
      pwndbg
      gcc
      hexedit
    ];
  };

  programs.zsh.enable = true;

  users.groups.julian = { gid = 1000; };
  # users.groups.wolfi = { gid = 1001; };

  # users.users.wolfi = {
  #   isNormalUser = true;
  #   description = "Wolfi";
  #   uid = 1001;
  #   group = "wolfi";
  #   shell = pkgs.fish;
  #   extraGroups = [ "networkmanager" "wheel" ];
  # };

  # Allow unfree packages
  nixpkgs.config.allowUnfree = true;
  nixpkgs.config.nvidia.acceptLicense = true;
  nixpkgs.config.permittedInsecurePackages = [
	"electron-24.8.6"
  ];

  environment.etc."manual-links/xfce4-notifyd".source =
    "${pkgs.xfce.xfce4-notifyd}/lib/xfce4/notifyd/xfce4-notifyd";
  environment.etc."manual-links/polkit-gnome-authentication-agent-1".source =
    "${pkgs.polkit_gnome}/libexec/polkit-gnome-authentication-agent-1";
  environment.pathsToLink = [ "/share/nix-direnv" ];
  environment.shells = with pkgs; [ zsh ]; # adds zsh to /etc/shells

  # List services that you want to enable:

  # Enable the OpenSSH daemon.
  services.openssh.enable = false;

  services.syncthing.enable = true;
  services.syncthing.user = "julian";
  services.syncthing.group = "julian";

  services.redshift.enable = true;

  services.flatpak.enable = true;

  # Smart card reader
  services.pcscd.enable = true;

  xdg.portal.enable = true;
  xdg.portal.extraPortals = [ pkgs.xdg-desktop-portal-gtk ];

  # services.emacs.enable = true;

  services.gnome.gnome-keyring.enable = true;

  # List packages installed in system profile. To search, run:
  # $ nix search wget

  environment.systemPackages = with pkgs; [
    # rfid reader tests
    openct
    opensc
    pcsclite
    pcsctools
    ccid
    libnfc

    xournalpp
    path-of-building

    xfce.thunar-archive-plugin
    zip
    p7zip
    imagemagick
    (pkgs.python3.withPackages python-packages)
    firefox
    thunderbird
    nextcloud-client
    qalculate-gtk
    libqalculate

    schildichat-desktop
    tdesktop

    tealdeer
    gcolor2 # gcolor3 does not work
    vim
    neovim
    # cups
    wget
    gparted
    emacs
    nodejs
    git
    dotter
    mc
    broot
    rnr # renaming tool in rust
    doublecmd
    mucommander
    tree
    alacritty
    wezterm
    kitty
    starship
    zsh
    bat
    ripgrep
    fd
    fzf
    topgrade
    cargo
    # rust-analyzer
    # rustfmt
    arandr
    wireguard-tools
    htop
    texlive.combined.scheme-full
    texlab
    usbutils # lsusb
    pciutils # lspci
    nixfmt
    gnome.gnome-keyring
    gnome.seahorse
    polkit_gnome
    nix-index
    python3
    black
    libnotify
    libclang
    libreoffice
    killall
    findutils
    xorg.xkill
    unzip
    vlc
    direnv
    nix-direnv
    handbrake
    nmap
    jq
    jdk
    # emacsPackages.clang-format # formatting for java
    gradle
    maven
    android-tools
    scrcpy
    dvdisaster
    mate.engrampa
    # toybox
    unixtools.procps
    qpdfview
    ffmpeg
    links2
    julia-bin
    xdg-desktop-portal-gtk
    discord
    lnav # log analyzing tool
    fdupes # find and delete duplicate files
    digikam
    dbeaver
    pavucontrol
    sqlite

    nomacs
    rocketchat-desktop
    # zotero
    inkscape
    pulseaudio
    gcolor3
    geogebra

    languagetool
    xclip
    okular # Pdf reader with many features, good for commenting documents
    evince # Simple pdf reader, good for focusing on document content
    pdfsam-basic # Split, merge, etc for pdfs

    poppler_utils # Pdf utils including pdfimages
    matlab # Using nix-matlab overlay defined in flake
    # sage

    gimp

    devbox
    parted
    drawio
    du-dust # Like du tree but better
    gnupg
    rpi-imager

    wine
    winetricks

    lf
    rustdesk
    realvnc-vnc-viewer
    calibre

    audible-cli

    cudaPackages.cudatoolkit 
    openmvg
    colmapWithCuda
  ];

  nixpkgs.overlays = [
    (self: super: {
      nix-direnv = super.nix-direnv.override { enableFlakes = true; };
    })
  ];

  fonts.fonts = with pkgs; [
    (nerdfonts.override { fonts = [ "FiraCode" ]; })
    font-awesome
    dejavu_fonts
    noto-fonts
    noto-fonts-cjk
    noto-fonts-emoji
    liberation_ttf
    fira-code
    fira-code-symbols
    source-code-pro
  ];

  virtualisation.docker.enable = true;

  virtualisation.virtualbox.host.enable = true;
  virtualisation.virtualbox.host.enableExtensionPack = true;
  virtualisation.virtualbox.guest.enable = true;
  virtualisation.virtualbox.guest.x11 = true;
  users.extraGroups.vboxusers.members = [ "julian" ];

  # Some programs need SUID wrappers, can be configured further or are
  # started in user sessions.
  # programs.mtr.enable = true;
  # programs.gnupg.agent = {
  #   enable = true;
  #   enableSSHSupport = true;
  # };

  # Open ports in the firewall.
  # networking.firewall.allowedTCPPorts = [ ... ];
  # networking.firewall.allowedUDPPorts = [ ... ];
  # Or disable the firewall altogether.
  # networking.firewall.enable = false;

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "22.11"; # Did you read the comment?

}
